# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from restish import http, resource

from ci_utils import amqp_utils, json_status, restish_utils

log = logging.getLogger(__name__)
WORKER_QUEUE = 'bsbuilder'


def _status():
    status = json_status.JSONStatus()
    status.add_rabbit_configured()
    status.add_rabbit_worker_health(WORKER_QUEUE)
    return restish_utils.json_ok(status.results)


def _build_source(ticket_id, subtickets, cancel_url,
                  series, ppa, archive, progress_trigger):
    params = {
        'ticket_id': ticket_id,
        'subtickets': subtickets,
        'cancel_url': cancel_url,
        'series': series,
        'ppa': ppa,
        'archive': archive,
        'progress_trigger': progress_trigger,
    }

    r = amqp_utils.progress_waiting(progress_trigger, params)
    if r:
        log.error('Unable to notify progress trigger, aborting build_source')
        return http.service_unavailable(body=r)

    r = amqp_utils.send(WORKER_QUEUE, json.dumps(params))
    if r:
        # send only returns something if it an error message
        r = http.service_unavailable(body=r)
    return r


class API(resource.Resource):
    @resource.child()
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return restish_utils.http_get_resource(_status)

    @resource.child()
    def build_source(self, request, segments):
        return restish_utils.http_post_resource(_build_source)
