# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock
import unittest

from bsbuilder import upload_package


class TestUploadPackage(unittest.TestCase):
    '''Tests for the upload_package module.'''

    subticket_list = [{
        'status': 'New',
        'current_workflow_step': 'New',
        'artifact': [{
            'id': 6,
            'type': 'SPU',
            'name': 'package_0.1_source.changes',
            'reference': 'https://swift.url/package_0.1_source.changes',
            'resource_uri': '/api/v1/fullsubticketartifact/6/'
        }, {
            'id': 5,
            'type': 'SPU',
            'name': 'package_0.1.tar.gz',
            'reference': 'https://swift.url/package_0.1.tar.gz',
            'resource_uri': '/api/v1/fullsubticketartifact/5/'
        }, {
            'id': 4,
            'type': 'SPU',
            'name': 'package_0.1.dsc',
            'reference': 'https://swift.url/package_0.1.dsc',
            'resource_uri': '/api/v1/fullsubticketartifact/4/'
        }],
        'assignee': 'user_id',
        'source_package_upload': {
            'version': '0.1',
            'sourcepackage': {
                'id': 1,
                'name': 'package',
                'resource_uri': '/api/v1/sourcepackage/1/'
            },
            'id': 2,
            'resource_uri': '/api/v1/spu/2/'
        },
        'id': 2,
        'resource_uri': '/api/v1/fullsubticket/2/'
    }]

    def _get_context_manager_mock(self):
        file_mock = mock.Mock()
        context_manager_mock = mock.Mock()
        context_manager_mock.__enter__ = mock.Mock()
        context_manager_mock.__enter__.return_value = file_mock
        context_manager_mock.__exit__ = mock.Mock()
        return context_manager_mock

    @mock.patch('bsbuilder.upload_package.create_temp_dir')
    @mock.patch('__builtin__.open')
    @mock.patch('bsbuilder.upload_package.get_url_contents')
    def test_get_source_files(self, url_mock, open_mock, temp_dir_mock):
        temp_dir_mock.return_value = 'mock_location'
        open_mock.return_value = self._get_context_manager_mock()
        file_list = []
        for item in self.subticket_list[0]['artifact']:
            file_list.append(item['name'])

        result = upload_package.get_source_files(self.subticket_list)
        self.assertEqual(1, len(result))
        spu = self.subticket_list[0]['source_package_upload']
        self.assertEqual(spu['sourcepackage']['name'], result[0]['name'])
        self.assertEqual(spu['version'], result[0]['version'])
        self.assertEqual('/api/v1/fullsubticket/2/', result[0]['resource'])
        self.assertEqual('mock_location', result[0]['location'])
        self.assertItemsEqual(file_list, result[0]['files'])
        url_mock.assert_called_with('https://swift.url/package_0.1.dsc')
