# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import unittest

import mock
import webtest

from bsbuilder import wsgi


class TestAPI(unittest.TestCase):
    '''Test to ensure the v1 API works.'''

    def setUp(self):
        super(TestAPI, self).setUp()
        self.app = webtest.TestApp(wsgi.app, relative_to='.')

    @mock.patch('ci_utils.amqp_utils.get_config')
    # Don't actually try to talk to a Rabbit server.
    @mock.patch('urllib2.build_opener')
    def testStatus(self, build_opener, get_config):
        get_config.return_value = None
        resp = self.app.get('/api/v1/status', status=200)
        data = json.loads(resp.body)
        expected = [{
            'label': 'rabbit configured',
            'value': False,
            'status': 'fail',
        }]
        self.assertListEqual(expected, data)

        config = mock.Mock()
        config.AMQP_HOST = 'bar'
        get_config.return_value = config
        resp = self.app.get('/api/v1/status', status=200)
        data = json.loads(resp.body)
        self.assertEqual(True, data[0]['value'])
        self.assertEqual('okay', data[0]['status'])

    @mock.patch('ci_utils.amqp_utils.get_config')
    def testBuildSourceUnconfigured(self, get_config):
        # ensure it fails when not configured
        get_config.return_value = None
        params = {
            'ticket_id': 1,
            'subtickets': 'foo',
            'cancel_url': None,
            'source_packages': 'foo',
            'series': 'foo',
            'ppa': 'foo',
            'archive': 'foo',
            'progress_trigger': 'foo',
        }
        resp = self.app.post_json('/api/v1/build_source', params, status=503)
        self.assertTrue('rabbitmq settings not available' in resp.body)

    def testBuildSourceBadParams(self):
        '''Ensure proper error message is returned for incorrect params.'''
        resp = self.app.post_json('/api/v1/build_source', {}, status=400)
        self.assertTrue('Missing required parameters' in resp.body)

    @mock.patch('ci_utils.amqp_utils.send')
    def testBuildSource(self, send):
        send.return_value = None
        params = {
            'ticket_id': 1,
            'subtickets': 'foo',
            'cancel_url': 'http://foo/api/json',
            'source_packages': 'foo',
            'series': 'foo',
            'ppa': 'foo',
            'archive': 'foo',
            'progress_trigger': 'foo',
        }
        r = self.app.post_json('/api/v1/build_source', params, status=204)
        self.assertEqual('', r.body)
