#!/usr/bin/env python

import distutils.dir_util
import grp
import json
import os
import pwd
import re
import shutil
import sys

import charmhelpers.core.hookenv
import charmhelpers.fetch
import charmhelpers.contrib.charmsupport.nrpe

from charmhelpers.core.hookenv import (
    config as config_get,
)

hooks = charmhelpers.core.hookenv.Hooks()
log = charmhelpers.core.hookenv.log


###############################################################################
# Global variables
###############################################################################
site_name = "ci-airlines.canonical.com"
site_path = "/srv/{}".format(site_name)

json_status_file = os.path.join(site_path, 'status_urls.json')

config_data = config_get()

web_uid = pwd.getpwnam('www-data')[2]
web_gid = grp.getgrnam('www-data')[2]


@hooks.hook('json_status-relation-joined')
@hooks.hook('json_status-relation-changed')
def json_status_relation_joined():
    url = charmhelpers.core.hookenv.relation_get('status-url')
    unit = charmhelpers.core.hookenv.remote_unit()
    log('status URL is: %s' % url)

    data = {}
    if os.path.exists(json_status_file):
        data = json.load(open(json_status_file))
    data[charmhelpers.core.hookenv.relation_id()] = {
        'url': url,
        'unit': unit,
    }

    with open(json_status_file, 'w') as f:
        json.dump(data, f)


@hooks.hook('json_status-relation-broken')
def json_status_relation_broken():
    if not os.path.exists(json_status_file):
        return

    data = json.load(open(json_status_file))
    rid = charmhelpers.core.hookenv.relation_id()
    if rid in data:
        del data[rid]
    with open(json_status_file, 'w') as f:
        json.dump(data, f)


@hooks.hook()
def install():
    if os.path.exists(site_path):
        shutil.rmtree(site_path)

    os.makedirs(site_path)

    # XXX: make these config options.
    os.chown(site_path, web_uid, web_gid)

    distutils.dir_util.copy_tree('files/webroot/', site_path)
    for root, dirs, files in os.walk(site_path):
        map(
            lambda file: os.chown(
                os.path.join(root, file), web_uid, web_gid
            ),
            files
        )
        map(
            lambda dir: os.chown(
                os.path.join(root, dir), web_uid, web_gid
            ),
            dirs
        )

    regex = re.compile(ur'^test.*\.html')
    for f in os.listdir(site_path):
        if regex.match(f):
            os.remove(os.path.join(site_path, f))


@hooks.hook()
def upgrade_charm():
    install()


if __name__ == "__main__":
    hooks.execute(sys.argv)
