# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import glanceclient
import keystoneclient.v2_0.client as ksclient
from urlparse import urlparse


class ImageStoreException(Exception):
    pass


class ImageStore(object):

    ImageStoreException = ImageStoreException

    def __init__(self, auth_config):
        GLANCE_API_VERSION = '1'
        self._setup_auth(auth_config)
        region = auth_config.get('auth_region')
        endpoints = self.keystone_client.service_catalog.get_endpoints(
            service_type='image', endpoint_type='publicURL')
        glance_endpoint = [
            x.get('publicURL') for x in endpoints.get('image')
            if x.get('region') == region
        ]
        if not glance_endpoint:
            raise ImageStoreException(
                'No valid glance endpoints found for the specified region. '
                'Check that the cloud credentials specified are valid')
        # Otherwise, there should be only one valid endpoint
        #Strip the version path in the url
        urlbits = urlparse(glance_endpoint[0])
        self.glance_endpoint = urlbits.scheme + '://' + urlbits.netloc
        self.glance = glanceclient.Client(
            GLANCE_API_VERSION, self.glance_endpoint,
            token=self.keystone_client.auth_token)

    def _setup_auth(self, config):
        try:
            self.keystone_client = ksclient.Client(
                auth_url=config['auth_url'],
                username=config['auth_user'],
                password=config['auth_password'],
                tenant_name=config['auth_tenant_name']
            )
        except (KeyError, ksclient.exceptions.Unauthorized,
                ksclient.exceptions.AuthorizationFailure):
            raise self.ImageStoreException(
                "Missing or invalid authentication info."
            )

    def image_create(self, filename, image, is_public=False,
                     disk_format='qcow2', container_format='bare'):
        with open(image) as imgdata:
            self.glance.images.create(
                name=filename, is_public=is_public, disk_format=disk_format,
                container_format=container_format, data=imgdata)
