#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils.image_store import ImageStore, ImageStoreException
from keystoneclient.exceptions import AuthorizationFailure
import unittest
import mock


class TestImageStore(unittest.TestCase):
    # Do not attempt to talk to a keystone server.
    @mock.patch('keystoneclient.httpclient.HTTPClient.authenticate',
                side_effect=AuthorizationFailure())
    def test_bad_auth(self, *args):
        auth_data = {
            'auth_url': 'https://keystone.canonistack.canonical.com:443/v2.0/',
            'auth_user': 'bad data',
            'auth_password': 'bad data',
            'auth_tenant_name': 'bad data'
        }
        with self.assertRaises(ImageStoreException):
            ImageStore(auth_data)

    def test_missing_auth(self):
        with self.assertRaises(ImageStoreException):
            ImageStore({})

    def test_bad_server(self):
        auth_data = {
            'auth_url': 'badurl',
            'auth_user': 'bad data',
            'auth_password': 'bad data',
            'auth_tenant_name': 'bad data'
        }
        with self.assertRaises(ImageStoreException):
            ImageStore(auth_data)

if __name__ == "__main__":
    unittest.main()
