# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import urllib2


class JSONStatus(object):
    def __init__(self):
        self.results = []

    def _add_result(self, name, value, status):
        self.results.append({
            'label': name,
            'value': value,
            'status': status,
        })

    def add_okay(self, name, value):
        self._add_result(name, value, 'okay')

    def add_fail(self, name, value):
        self._add_result(name, value, 'fail')

    def add_true_false(self, name, value, condition):
        '''adds an 'okay' result if condition is true, 'fail' otherwise'''
        if condition:
            self.add_okay(name, value)
        else:
            self.add_fail(name, value)

    def add_rabbit_configured(self):
        '''adds a result status for status of rabbit-configuration'''
        # we don't want to import this for all users of the API. Some
        # may not install rabbit (eg the ppa-assigner)
        from ci_utils import amqp_utils
        if amqp_utils.get_config() is not None:
            self.add_okay('rabbit configured', True)
        else:
            self.add_fail('rabbit configured', False)

    def add_rabbit_worker_health(self, queue):
        '''checks if there are any workers attached to the queue.'''
        from ci_utils import amqp_utils
        config = amqp_utils.get_config()
        if config is None:
            # we already report rabbit isn't configured, so no sense adding
            # another failure for this
            return

        mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        base = 'http://{}:55672'.format(config.AMQP_HOST)
        # NOTE: juju leaves this guest/guest and doesn't seem to expose
        # anything in the relation to override it
        mgr.add_password(None, base, 'guest', 'guest')
        handler = urllib2.HTTPBasicAuthHandler(mgr)

        try:
            resp = urllib2.build_opener(handler).open(base + '/api/queues')
            content = resp.read()
            data = json.loads(content)
            consumers = None
            for q in data:
                if q['name'] == queue:
                    consumers = q['consumers']
                    break
            if consumers is None:
                self.add_fail('workers-online', 'no queue defined')
            else:
                self.add_true_false('workers-online', consumers, consumers > 0)
        except:
            self.add_fail('workers-online', 'unable to check')
