# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from ci_utils import data_store

auth_config_valid = {
    'auth_url': 'http://example.com',
    'auth_user': 'user',
    'auth_password': 'pass',
    'auth_tenant_name': 'tenant',
    'auth_region': 'region',
}

auth_config_invalid = {}

filename = 'myproject_1.0-1_source.changes'


class TestDataStore(unittest.TestCase):
    def test_valid_auth_config(self):
        data_store.DataStore.validate_auth_config(auth_config_valid)

    def test_invalid_auth_config(self):
        with self.assertRaises(data_store.DataStoreException):
            data_store.DataStore.validate_auth_config(auth_config_invalid)

    def test_get_file_name(self):
        path = '/home/ubuntu/myproject/{}'.format(filename)
        resp = data_store._get_file_name(path)
        self.assertEquals(resp, filename)

    def test_get_relativepath_file_name(self):
        path = '../../myproject/{}'.format(filename)
        resp = data_store._get_file_name(path)
        self.assertEquals(resp, filename)

    def test_get_file_name_same_directory(self):
        path = '{}'.format(filename)
        resp = data_store._get_file_name(path)
        self.assertEquals(resp, filename)

if __name__ == "__main__":
    unittest.main()
