# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import unittest

import mock

from ci_utils.json_status import JSONStatus


class TestRabbitStatus(unittest.TestCase):
    @mock.patch('ci_utils.amqp_utils.get_config')
    def testNoConfig(self, get_config):
        get_config.return_value = None
        status = JSONStatus()
        status.add_rabbit_worker_health('foo')
        self.assertEqual([], status.results)

    @mock.patch('ci_utils.amqp_utils.get_config')
    @mock.patch('urllib2.build_opener')
    def testCantReach(self, opener, get_config):
        opener.side_effect = RuntimeError('foo')
        status = JSONStatus()
        status.add_rabbit_worker_health('foo')
        expected = {
            'status': 'fail',
            'value': 'unable to check',
            'label': 'workers-online'
        }
        self.assertEqual([expected], status.results)

    @mock.patch('ci_utils.amqp_utils.get_config')
    @mock.patch('urllib2.build_opener')
    def _testRabbitHealth(self, queue, data, opener, get_config):
        fdfake = mock.Mock()
        fdfake.read.return_value = json.dumps(data)
        con = mock.Mock()
        con.open.return_value = fdfake
        opener.return_value = con
        status = JSONStatus()
        status.add_rabbit_worker_health(queue)
        return status.results

    def testWorkers(self):
        data = [
            {'name': 'bla', 'consumers': 2},
            {'name': 'foo', 'consumers': 1},
            {'name': 'bar', 'consumers': 1},
        ]
        expected = {
            'status': 'okay',
            'value': 1,
            'label': 'workers-online'
        }
        results = self._testRabbitHealth('foo', data)
        self.assertEqual([expected], results)

    def testNoWorkers(self):
        data = [
            {'name': 'bla', 'consumers': 2},
            {'name': 'foo', 'consumers': 0},
            {'name': 'bar', 'consumers': 1},
        ]
        expected = {
            'status': 'fail',
            'value': 0,
            'label': 'workers-online'
        }
        results = self._testRabbitHealth('foo', data)
        self.assertEqual([expected], results)

    def testNoQueue(self):
        data = [
            {'name': 'bla', 'consumers': 2},
            {'name': 'bar', 'consumers': 1},
        ]
        expected = {
            'status': 'fail',
            'value': 'no queue defined',
            'label': 'workers-online'
        }
        results = self._testRabbitHealth('foo', data)
        self.assertEqual([expected], results)
