# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import sys
import urllib2
import yaml

from ci_utils import data_store

log = logging.getLogger()


HEADERS = {'content-type': 'application/json'}
API_URL = '/api/v1/'
TICKET_BASE = API_URL + 'ticket/'
SPU_BASE = API_URL + 'spu/'
SOURCEPACKAGE_BASE = API_URL + 'sourcepackage/'
ARTIFACT_BASE = API_URL + 'subticketartifact/'
SUBTICKET_BASE = API_URL + 'subticket/'

CI_URL = None
AUTH_CONFIG = None


class InputError(Exception):
    """Exception raised for errors in the input."""


def load_config(config_file):
    global CI_URL
    global AUTH_CONFIG
    try:
        with open(config_file) as fp:
            AUTH_CONFIG = yaml.safe_load(fp)
            CI_URL = AUTH_CONFIG['ci_url']
        data_store.DataStore.validate_auth_config(AUTH_CONFIG)
    except IOError:
        r = ['ci_url', 'auth_url', 'auth_user', 'auth_password',
             'auth_region', 'auth_tenant_name']
        error_msg = ("ERROR: Missing YAML config in {}:\n {}"
                     "\n\nThis can be created by running: {} auto_config")
        sys.exit(error_msg.format(config_file, "\n ".join(r), sys.argv[0]))
    except KeyError as exc:
        error_msg = "ERROR: missing config in {}: '{}' is not set."
        sys.exit(error_msg.format(config_file, exc.message))
    except yaml.parser.ParserError as exc:
        error_msg = ("ERROR: Problem parsing config file in {}: {}, {}. "
                     "Missing quotes?")
        sys.exit(error_msg.format(config_file, exc.context, exc.problem))
    except data_store.DataStoreException as exc:
        error_msg = ("ERROR: data-store: {}")
        sys.exit(error_msg.format(exc))


def post(url, data, headers=HEADERS, patch=False):
    req = urllib2.Request(url=url, data=json.dumps(data), headers=HEADERS)
    f = urllib2.urlopen(req)
    return f.info().dict['location']


def patch(url, data, headers=HEADERS):
    req = urllib2.Request(url=url, data=json.dumps(data), headers=HEADERS)
    req.get_method = lambda: 'PATCH'
    urllib2.urlopen(req)


def get(url):
    f = urllib2.urlopen(url)

    json_data = f.read()
    return json.loads(json_data)


def parse_id(location):
    try:
        id = location.split('/')[6]
        int(id)
    except ValueError, exc:
        raise exc
    return id


def get_sourcepackage_uri(sourcepackage):
    url = CI_URL + SOURCEPACKAGE_BASE + '?name__iexact=' + sourcepackage
    f = urllib2.urlopen(url)
    json_data = f.read()
    data = json.loads(json_data)
    try:
        sourcepackage_uri = data['objects'][0]['resource_uri']
        return sourcepackage_uri
    except IndexError:
        # Package not found.
        log.info("Source package not found: %s" % sourcepackage)
        return ''


def assert_valid_package_list(package_list_str):
    if package_list_str and " " in package_list_str:
        raise InputError("Added and removed packages have to be "
                         "comma-separated lists with no spaces.")
