#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import mock
import imp

from ci_libs.ticket import SubTicket, Ticket
from tests import TestCaseWithGnupg


class SubTicketTestCase(TestCaseWithGnupg):

    def setUp(self):
        super(SubTicketTestCase, self).setUp()
        self.changes = os.path.join(os.path.dirname(__file__),
                                    'data/foobar_0.1-1_source.changes')

    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/sourcepackage/4/')
    def test_create_sourcepackage(self, mock_post):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=4)
        new_subticket._parse_changes(self.changes)
        sourcepackage_uri = new_subticket._create_sourcepackage()
        mock_post.assert_called_once()
        self.assertEqual(sourcepackage_uri, '/api/v1/sourcepackage/4/')

    @mock.patch('ci_libs.utils.get_sourcepackage_uri',
                return_value='/api/v1/sourcepackage/5/')
    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/spu/38/')
    def test_create_spu(self, mock_post, mock_sp_uri):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=3)
        new_subticket._parse_changes(self.changes)
        new_subticket._create_spu()

    @mock.patch('ci_libs.utils.get_sourcepackage_uri', return_value='')
    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/sourcepackage/5/')
    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/spu/39/')
    def test_create_spu_sourcepackage_not_found(self, mock_spu_post, mock_post,
                                                mock_sp_uri):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=3)
        new_subticket._parse_changes(self.changes)
        new_subticket._create_spu()

    @mock.patch(
        'ci_libs.utils.post',
        return_value='http://www.example.com/api/v1/subticketartifact/38/',
    )
    def test_create_artifact(self, mock_post):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=2)
        file = 'foobar_source.changes'
        location = 'http://www.example.com/path/to/foobar_source.changes'
        new_subticket._create_artifact(file, location)
        mock_post.assert_called_once()

    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/subticket/1/')
    def test_create_subticket(self, mock_post):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=1)
        new_subticket._create_subticket()


class TicketTestCase(TestCaseWithGnupg):

    def setUp(self):
        super(TicketTestCase, self).setUp()
        self.cli = imp.load_source(
            "ubuntu-ci",
            os.path.join(os.path.dirname(__file__), "../ubuntu-ci")
        )

    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/ticket/38/')
    def test_create_ticket(self, mock_post):
        changes = os.path.join(os.path.dirname(__file__),
                               'data/foobar_0.1-1_source.changes')
        args = ['create_ticket', '-t', 'New feature', '-b', '1234', '-o',
                'someone@example.com', '-d', 'This is a cool new feature',
                '-a', 'foo', '-r', 'baz', '-s', changes]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket._create_ticket(args)
