# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from restish import http, resource

from ci_utils import amqp_utils, json_status, restish_utils

log = logging.getLogger(__name__)
WORKER_QUEUE = 'imagebuilder'


def _build_image(base_image, ppa_list, package_list, progress_trigger,
                 ticket_id, cancel_url):
    params = {
        'image': base_image['url_list'][0],
        'ppa_list': ppa_list,
        'series': base_image['series'],
        'package_list': package_list,
        'progress_trigger': progress_trigger,
        'ticket_id': ticket_id,
        'cancel_url': cancel_url,
    }

    r = amqp_utils.progress_waiting(progress_trigger, params)
    if r:
        log.error('Unable to notify progress trigger, aborting build_image')
        return http.service_unavailable(body=r)

    r = amqp_utils.send(WORKER_QUEUE, json.dumps(params))
    if r:
        r = http.service_unavailable(body=r)
    return r


def _status():
    status = json_status.JSONStatus()
    status.add_rabbit_configured()
    status.add_rabbit_worker_health(WORKER_QUEUE)
    return restish_utils.json_ok(status.results)


class API(resource.Resource):

    @resource.child('status')
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return restish_utils.http_get_resource(_status)

    @resource.child()
    def build_image(self, request, segments):
        log.debug('[build_image]: %s %s', request.url,
                  json.dumps(request.body))
        return restish_utils.http_post_resource(_build_image)

    def _return_json(self, request):
        try:
            params = json.loads(request.body)
        except ValueError:
            return http.bad_request()
        return http.ok([('Content-Type', 'application/json')],
                       json.dumps(params))
