#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import amqp_utils, amqp_worker

from imagebuilder.cloud_image import modify_image


class ImageBuildWorker(amqp_worker.AMQPWorker):
    def __init__(self):
        super(ImageBuildWorker, self).__init__('image_build')

    def handle_request(self, log, params):
        image = params['image']
        repos = params['ppa_list']
        series = params['series']
        packages = params['package_list']
        trigger = params['progress_trigger']
        ds_name = params['ticket_id'] + '-image'
        ds = self._create_data_store(ds_name)

        def status_cb(msg):
            log.info(msg)
            amqp_utils.progress_update(trigger, {'message': msg})

        image_id, location = modify_image(image, repos, series, packages,
                                          ds, status_cb)
        retval = {'image_id': image_id}
        self._create_artifact(image_id, location, retval, type='IMAGE')
        return amqp_utils.progress_completed, retval


if __name__ == '__main__':
    ImageBuildWorker().main('imagebuilder')
