# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock
import unittest
import webtest

from imagebuilder import wsgi


class TestAPI(unittest.TestCase):
    def setUp(self):
        super(TestAPI, self).setUp()
        self.app = webtest.TestApp(wsgi.app, relative_to='.')

    @mock.patch('ci_utils.amqp_utils.get_config')
    def test_status(self, get_config):
        get_config.return_value = None
        resp = self.app.get('/api/v1/status')
        self.assertEqual(200, resp.status_code)

    @mock.patch('ci_utils.amqp_utils.send')
    def test_build_image(self, send):
        send.return_value = None
        args = {
            'base_image': {
                'url_list': ['http://foo.bar/image.img'],
                'series': 'trusty',
                'image_type': 'cloud'
            },
            'ppa_list': ['ppa:fakeproject/test'],
            'package_list': ['fakepackage'],
            'progress_trigger': 'triggername',
            'ticket_id': '1',
            'cancel_url': None,
        }
        resp = self.app.post_json('/api/v1/build_image', args)
        self.assertEqual('', resp.body)
        self.assertEqual(204, resp.status_code)

    @mock.patch('ci_utils.amqp_utils.get_config')
    def test_build_image_noconfig(self, get_config):
        get_config.return_value = None
        args = {
            'base_image': {
                'url_list': ['http://foo.bar/image.img'],
                'series': 'trusty',
                'image_type': 'cloud'
            },
            'ppa_list': ['ppa:fakeproject/test'],
            'package_list': ['fakepackage'],
            'progress_trigger': 'triggername',
            'ticket_id': '1',
            'cancel_url': 'http://foo/api/json',
        }
        resp = self.app.post_json('/api/v1/build_image', args, status=503)
        self.assertTrue('rabbitmq settings not available' in resp.body)

    def test_bad_build_image(self):
        resp = self.app.post('/api/v1/build_image', '{}', status=400)
        self.assertTrue('Missing required parameters' in resp.body)
        self.assertEqual(400, resp.status_code)
