# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess
import unittest

import mock

from imagebuilder import cloud_image


class TestCloudImage(unittest.TestCase):
    def setUp(self):
        super(TestCloudImage, self).setUp()

    @mock.patch('imagebuilder.cloud_image.launchpad')
    def test_parse_ppas(self, launchpad):
        launchpad.get_lp_user_name.return_value = 'foo'

        lp = mock.MagicMock()
        ppa = mock.Mock()
        ppa.name = 'foo'
        ppa.signing_key_fingerprint = 'foofp'
        ppa.private = True
        lp.people['foo'].getPPAByName.return_value = ppa
        lp.people['foo'].getArchiveSubscriptionURL.return_value = 'foo_url'

        series = 'precise'
        deblines, keys = cloud_image._parse_ppas(lp, ['ppa:/foo/bar'], series)
        self.assertEqual({'foo': 'foofp'}, keys)
        expected = ['deb foo_url precise main', 'deb-src foo_url precise main']
        self.assertListEqual(expected, deblines)

    @mock.patch('imagebuilder.cloud_image._chrooted')
    def test_apt_get_update_fails(self, chrooted):
        chrooted.side_effect = subprocess.CalledProcessError(1, 'cmd', 'out')
        with self.assertRaises(subprocess.CalledProcessError):
            cloud_image._apt_get_update('/blah')

    @mock.patch('imagebuilder.cloud_image._chrooted')
    def test_apt_get_update_retries(self, chrooted):
        '''ensure it can apt-get update works after retrying'''
        chrooted.side_effect = [
            subprocess.CalledProcessError(1, 'cmd', 'out'),
            subprocess.CalledProcessError(1, 'cmd', 'out'),
            None,
        ]
        cloud_image._apt_get_update('/blah')
        self.assertEqual(3, chrooted.call_count)

    @mock.patch('imagebuilder.cloud_image.launchpad')
    @mock.patch('imagebuilder.cloud_image._chrooted')
    @mock.patch('__builtin__.open')
    def test_setup_ppas(self, mopen, chroot, lp):
        status_cb = mock.Mock()
        series = 'saucy'
        ppas = ['ppa:user/foo', 'ppa:team/bar']
        mopen.return_value = mock.MagicMock(spec=file)
        fd = mopen.return_value.__enter__.return_value
        cloud_image._setup_ppas('/chroot', ppas, series, status_cb)
        self.assertTrue(fd.write.call_count)
        # todo look at chroot calls
