# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from restish import http, resource

from ci_utils import jenkins_utils, restish_utils

log = logging.getLogger(__name__)


def _status():
    config = jenkins_utils.get_config()
    return restish_utils.json_ok({
        'jenkins_configured': config is not None,
    })


def _submit_request(ticket, series, source_packages, progress_trigger):
    config = jenkins_utils.get_config()
    params = json.dumps({
        'ticket': ticket,
        'source_packages': source_packages,
        'series': series,
        'progress_trigger': progress_trigger,
    })
    params = params.replace(' ', '')
    log.debug('Params: {}'.format(params))

    job_parameters = {'request_id': ticket,
                      'request_parameters': params}

    result = jenkins_utils.trigger_build(config, job_parameters)
    if result:
        # send only returns something if it an error message
        result = http.service_unavailable(body=result)
    return result


class API(resource.Resource):
    @resource.child()
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return restish_utils.http_get_resource(_status)

    @resource.child()
    def submit_request(self, request, segments):
        log.debug('[submit_request]: %s %s', request.url, str(segments))
        return restish_utils.http_post_resource(_submit_request)
