# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import unittest

import mock
import webtest

from lander import wsgi


class TestAPI(unittest.TestCase):
    '''Test to ensure the v1 API works.'''

    def setUp(self):
        super(TestAPI, self).setUp()
        self.app = webtest.TestApp(wsgi.app, relative_to='.')
        self.config = {'lander_hostname': 'localhost',
                       'lander_port': '8080',
                       'lander_username': 'lander-bot',
                       'lander_password': 'lander-bot'}

    @mock.patch('ci_utils.jenkins_utils.get_config')
    def test_status_config_ok(self, get_config_mock):
        '''Verify a valid jenkins config returns true.'''
        get_config_mock.return_value = self.config
        response = self.app.get('/api/v1/status', status=200)
        data = json.loads(response.body)
        self.assertTrue(data['jenkins_configured'])

    @mock.patch('ci_utils.jenkins_utils.get_config')
    def test_status_config_failed(self, get_config_mock):
        '''Verify a failed jenkins config returns false.'''
        get_config_mock.return_value = None
        response = self.app.get('/api/v1/status', status=200)
        data = json.loads(response.body)
        self.assertFalse(data['jenkins_configured'])

    @mock.patch('ci_utils.jenkins_utils.get_config')
    @mock.patch('ci_utils.jenkins_utils.trigger_build')
    def test_submit_request_ok(self, trigger_build_mock, get_config_mock):
        '''Verify that a request is successfully handled.'''
        get_config_mock.return_value = self.config
        trigger_build_mock.return_value = None
        params = {
            'ticket': '1234',
            'series': 'trusty',
            'source_packages': [
                'http://data_store_url/foobar_0.1-1.changes',
                'http://data_store_url/foobar_0.1-1.dsc',
                'http://data_store_url/foobar_0.1-1.debian.tar.gz'],
            'progress_trigger': 'progress_trigger_handle'}
        response = self.app.post_json('/api/v1/submit_request', params,
                                      status=204)
        self.assertEqual('', response.body)
