# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import urllib2

from optparse import make_option

from django.core.management.base import BaseCommand

from ppa_assigner import launchpad
from ppa_assigner.models import PPA


class Command(BaseCommand):
    help = 'Utilities to setup an integration test against launchpad.'
    option_list = BaseCommand.option_list + (
        make_option('--reserve', dest='reserve', action='store_true',
                    help='Find a free PPA in the pool and reserve it.'),
        make_option('--make-dirty', dest='make_dirty', type=str,
                    help='Put content in the PPA and mark it dirty'),
        make_option('--source_name', dest='source_name',
                    default='apport',
                    help='''Source package name to copy into our test ppa.
                         default=%(default)s'''),
        make_option('--source_version', dest='source_version',
                    default='2.12.7-0ubuntu2',
                    help='''Source package version to copy into our test ppa.
                         default=%(default)s'''),
        make_option('--pocket', dest='pocket', default='Release',
                    help='Pocket to copy package in. default=%(default)s'),
        make_option(
            '--archive', dest='archive',
            default='https://api.launchpad.net/1.0/ubuntu/+archive/primary',
            help='Archive for source package. defafult=%(default)s'),
    )

    def handle(self, *args, **options):
        level = logging.INFO
        if int(options['verbosity']) > 1:
            level = logging.DEBUG
        logging.basicConfig(level=level)

        if options['reserve']:
            self._reserve()
        if options['make_dirty']:
            self._make_dirty(options)

    def _reserve(self):
        p = PPA.reserve(0)
        print(p.name)

    def _make_dirty(self, options):
        ppa = options['make_dirty']
        name = options['source_name']
        ver = options['source_version']
        pocket = options['pocket']
        archive = options['archive']
        try:
            launchpad._sync_source(ppa, name, ver, pocket, archive)
            p = PPA.objects.get(name=ppa)
            p.state = PPA.DIRTY
            p.save()
        except urllib2.HTTPError as e:
            print('HTTPError: %s:\n%s' % (e.reason, e.read()))
