# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
from restish import (
    http,
    resource,
)

from ci_utils import (
    amqp_utils,
    json_status,
    restish_utils,
)

import tstrun

log = logging.getLogger(__name__)


def _status():
    status = json_status.JSONStatus()
    status.add_rabbit_configured()
    status.add_rabbit_worker_health(tstrun.internal_rabbit_queue)
    return restish_utils.json_ok(status.results)


def _test_image(ticket_id, cancel_url, image_id, package_list,
                progress_trigger):
    params = dict(ticket_id=ticket_id,
                  cancel_url=cancel_url,
                  image_id=image_id,
                  package_list=package_list,
                  progress_trigger=progress_trigger)
    res = amqp_utils.progress_waiting(progress_trigger, params)
    if res:
        log.error('Unable to notify progress queue, aborting test run')
        return http.service_unavailable(body=res)
    res = amqp_utils.send(tstrun.internal_rabbit_queue, json.dumps(params))
    if res:
        res = http.service_unavailable(body=res)
    return res


class API(resource.Resource):

    @resource.child()
    def test_image(self, request, segments):
        # FIXME: Find how to redirect log, it pollutes test output and
        # redirecting stdout/stderr in the tests is overkill -- vila 2014-01-07
#        log.debug('[test_image]: %s %s', request.url, str(segments))
        return restish_utils.http_post_resource(_test_image)

    @resource.child()
    def status(self, request, segments):
        return restish_utils.http_get_resource(_status)
