# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from cStringIO import StringIO
import testtools
import unittest


import run_test


class TestParseSummary(unittest.TestCase):

    def assertParsedLine(self, expected, line):
        self.assertEqual(expected,
                         run_test.parse_summary_line(line))

    def test_pass(self):
        self.assertParsedLine(('dsc0t-upstream', 'PASS'),
                              'dsc0t-upstream       PASS')

    def test_fail(self):
        self.assertParsedLine(('dsc0t-upstream', 'FAIL'),
                              'dsc0t-upstream       FAIL')

    def test_fail_with_exit_code(self):
        self.assertParsedLine(
            ('dsc0t-local-provider', 'FAIL'),
            'dsc0t-local-provider FAIL non-zero exit status 1')

    def test_skip(self):
        self.assertParsedLine(('* no tests in this package', 'SKIP'),
                              '*                    SKIP '
                              'no tests in this package')

    def test_empty(self):
        self.assertParsedLine(('', 'error'), '')

    def test_simply_bogus(self):
        self.assertParsedLine(('asingleword', 'error'),
                              'asingleword')

    def test_bogus(self):
        # A completely bogus line appears out of nowhere
        self.assertParsedLine(('stranger', 'in the night'),
                              'stranger in the night')


class TestProcessTestResult(unittest.TestCase):

    def fake_result_for_dep8_test(self, status, name):
        out = StringIO()
        # We diverge from the implementation that uses a
        # subunit.TestProtocolClient to produce a subunit stream. Instead, we
        # use a testtools.TestResult to check that the desired input triggers
        # the right kind of test in the output. Testing that subunit and
        # testtools properly communicate is covered in uci-tests. The reason we
        # do that is that it's simpler to use a testtools.TestResult than to
        # fix subunit to properly set the test result attributes that summarize
        # the run (.errors, .failures, etc). It's also easier to test this
        # attributes than to parse the subunit output to calculate them.
        res = testtools.TestResult(out)
        res.startTestRun()
        run_test.process_test_result('fake-package', res, name, status)
        res.stopTestRun()
        return res

    def test_passs(self):
        res = self.fake_result_for_dep8_test('PASS', 'xx')
        self.assertTrue(res.wasSuccessful())
        self.assertEqual(0, len(res.errors))
        self.assertEqual(0, len(res.failures))
        self.assertEqual(0, len(res.skipped))

    def test_fail(self):
        res = self.fake_result_for_dep8_test('FAIL', 'xx')
        self.assertFalse(res.wasSuccessful())
        self.assertEqual(1, len(res.failures))

    def test_skip(self):
        res = self.fake_result_for_dep8_test('SKIP', 'xx')
        self.assertTrue(res.wasSuccessful())
        self.assertEqual(1, len(res.skip_reasons))

    def test_error(self):
        res = self.fake_result_for_dep8_test('foo', 'bar')
        self.assertFalse(res.wasSuccessful())
        self.assertEqual(1, len(res.errors))
