#!/usr/bin/env python

import json
import os
import unittest

import httplib2

from tests import deployers


class TestTestRunner(deployers.DeployerTest):
    """Integration tests for image builder service run on a juju deployment"""

    deployer_cfg = os.path.join(deployers.DEPLOYER_DIR, 'image-builder.yaml')

    def get_status(self):
        url = 'http://{}:{}/api/v1/status'.format(
            *self.get_ip_and_port('imagebuild-restish'))
        client = httplib2.Http()
        resp, content = client.request(
            url, 'GET',
        )
        return resp, content

    def post_build_image(self, body):
        url = 'http://{}:{}/api/v1/build_image'.format(
            *self.get_ip_and_port('imagebuild-restish'))
        client = httplib2.Http()
        resp, content = client.request(
            url, 'POST',
            headers={'Content-Type': 'application/json'},
            body=json.dumps(body),
        )
        return resp, content

    def test_build_good_args(self):
        body = {
            'base_image': {
                'url_list': ['http://foo.bar/image.img'],
                'series': 'trusty',
                'image_type': 'cloud'
            },
            'ppa_list': ['ppa:fakeproject/test'],
            'package_list': ['fakepackage'],
            'progress_trigger': 'triggername'
        }
        resp, content = self.post_build_image(body)
        self.assertEqual('204', resp['status'])
        self.assertEqual('', content)

    def test_build_no_args(self):
        body = {}
        resp, content = self.post_build_image(body)
        self.assertEqual('400', resp['status'])
        self.assertIn('Missing required parameters', content)

    def test_get_status(self):
        resp, content = self.get_status()
        self.assertEqual('200', resp['status'])
        data = json.loads(content)
        self.assertEqual(True, data['rabbit_configured'])

    def test_worker_running(self):
        '''ensure the rabbit worker is deployed and running'''
        self.assert_job_running('imagebuild_worker', 'imagebuild-worker')

if __name__ == "__main__":
    unittest.main()
