# Ubuntu Continuous Integration Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from model_mommy import mommy
from model_mommy.recipe import Recipe, seq
from ci_utils import ticket_states
from ci_utils.tastypie.test import TastypieTestCase
from project.models import SourcePackage
from ticket.models import (get_enum_title, SubTicketWorkflowStep,
                           SubTicketWorkflowStepStatus, TicketWorkflowStep,
                           TicketWorkflowStepStatus)
from django.conf import settings

sourcepackage_recipe = Recipe(SourcePackage, name=seq('foobar'))


class TicketReadAPITest(TastypieTestCase):

    def setUp(self):
        super(TicketReadAPITest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.sourcepackage = sourcepackage_recipe.make()
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.subticket = mommy.make('SubTicket', ticket=self.ticket,
                                    source_package_upload=self.spu)
        self.artifact_1 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket)
        self.artifact_2 = mommy.make('TicketArtifact', ticket=self.ticket)
        self.maxDiff = None

    def test_get_artifact_api_subticket(self):
        obj = self.getResource('subticketartifact/')
        self.assertEqual(obj['objects'][0], {
            u'name': unicode(self.artifact_1.name),
            u'reference': unicode(self.artifact_1.reference),
            u'subticket': {
                u'status': unicode(get_enum_title(self.subticket.status,
                                   SubTicketWorkflowStepStatus)),
                u'current_workflow_step': unicode(get_enum_title(
                    self.subticket.current_workflow_step,
                    SubTicketWorkflowStep)),
                u'assignee': unicode(self.subticket.assignee),
                u'source_package_upload': {
                    u'version': unicode(self.spu.version),
                    u'sourcepackage': {
                        u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                            self.sourcepackage.pk),
                        u'id': self.sourcepackage.pk,
                        u'name': unicode(self.sourcepackage.name)},
                    u'id': self.spu.pk,
                    u'resource_uri':
                    u'/api/v1/spu/{0}/'.format(self.spu.pk)},
                u'ticket': {
                    u'added_binaries': self.ticket.added_binaries,
                    u'status': unicode(get_enum_title(self.ticket.status,
                                       TicketWorkflowStepStatus)),
                    u'created': unicode(self.ticket.created.strftime(
                        settings.TEST_DATETIME_FORMAT)),
                    u'current_workflow_step': unicode(get_enum_title(
                        self.ticket.current_workflow_step,
                        TicketWorkflowStep)),
                    u'description': unicode(self.ticket.description),
                    u'title': unicode(self.ticket.title),
                    u'bug_id': self.ticket.bug_id,
                    u'owner': unicode(self.ticket.owner),
                    u'removed_binaries': self.ticket.removed_binaries,
                    u'base_image': unicode(self.ticket.base_image),
                    u'id': self.ticket.pk,
                    u'master_ppa': unicode(self.ticket.master_ppa),
                    u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                        self.ticket.pk),
                    u'series': unicode(self.ticket.series),
                    u'updated': unicode(self.ticket.updated.strftime(
                        settings.TEST_DATETIME_FORMAT))},
                u'id': self.subticket.pk,
                u'resource_uri': u'/api/v1/subticket/{0}/'.format(
                    self.subticket.pk)},
            u'type': unicode(self.artifact_1.type),
            u'id': self.artifact_1.pk,
            u'resource_uri': u'/api/v1/subticketartifact/{0}/'.format(
                self.artifact_1.pk),
        })

    def test_get_artifact_api_ticket(self):
        obj = self.getResource('ticketartifact/')
        self.assertEqual(obj['objects'][0], {
            u'name': unicode(self.artifact_2.name),
            u'reference': unicode(self.artifact_2.reference),
            u'ticket': {
                u'added_binaries': self.ticket.added_binaries,
                u'status': unicode(get_enum_title(self.ticket.status,
                                   TicketWorkflowStepStatus)),
                u'created': unicode(self.ticket.created.strftime(
                    settings.TEST_DATETIME_FORMAT)),
                u'current_workflow_step': unicode(get_enum_title(
                    self.ticket.current_workflow_step,
                    TicketWorkflowStep)),
                u'description': unicode(self.ticket.description),
                u'title': unicode(self.ticket.title),
                u'bug_id': self.ticket.bug_id,
                u'owner': unicode(self.ticket.owner),
                u'removed_binaries': self.ticket.removed_binaries,
                u'base_image': unicode(self.ticket.base_image),
                u'id': self.ticket.pk,
                u'master_ppa': unicode(self.ticket.master_ppa),
                u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                    self.ticket.pk),
                u'series': unicode(self.ticket.series),
                u'updated': unicode(self.ticket.updated.strftime(
                    settings.TEST_DATETIME_FORMAT))},
            u'type': unicode(self.artifact_2.type),
            u'id': self.artifact_2.pk,
            u'resource_uri': u'/api/v1/ticketartifact/{0}/'.format(
                self.artifact_2.pk),
        })

    def test_get_subticket_api(self):
        obj = self.getResource('subticket/')
        self.assertEqual(obj['objects'][0], {
            u'assignee': unicode(self.subticket.assignee),
            u'current_workflow_step': unicode(get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep)),
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/subticket/{0}/'.format(
                self.subticket.pk),
            u'source_package_upload': {
                u'version': unicode(self.spu.version),
                u'sourcepackage': {
                    u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                        self.sourcepackage.pk),
                    u'id': self.sourcepackage.pk,
                    u'name': unicode(self.sourcepackage.name)},
                u'id': self.spu.pk,
                u'resource_uri':
                u'/api/v1/spu/{0}/'.format(self.spu.pk)},
            u'status': unicode(get_enum_title(self.subticket.status,
                               SubTicketWorkflowStepStatus)),
            u'ticket': {
                u'added_binaries': self.ticket.added_binaries,
                u'base_image': unicode(self.ticket.base_image),
                u'bug_id': self.ticket.bug_id,
                u'created': unicode(self.ticket.created.strftime(
                    settings.TEST_DATETIME_FORMAT)),
                u'current_workflow_step': unicode(get_enum_title(
                    self.ticket.current_workflow_step,
                    TicketWorkflowStep)),
                u'description': unicode(self.ticket.description),
                u'id': self.ticket.pk,
                u'master_ppa': unicode(self.ticket.master_ppa),
                u'owner': unicode(self.ticket.owner),
                u'removed_binaries': self.ticket.removed_binaries,
                u'status': unicode(get_enum_title(self.ticket.status,
                                   TicketWorkflowStepStatus)),
                u'title': unicode(self.ticket.title),
                u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                    self.ticket.pk),
                u'series': unicode(self.ticket.series),
                u'updated': unicode(self.ticket.updated.strftime(
                    settings.TEST_DATETIME_FORMAT))},
        })

    def test_get_ticket_api(self):
        obj = self.getResource('ticket/')
        self.assertEqual(obj['objects'][0], {
            u'added_binaries': self.ticket.added_binaries,
            u'status': unicode(get_enum_title(self.ticket.status,
                               TicketWorkflowStepStatus)),
            u'created': unicode(self.ticket.created.strftime(
                settings.TEST_DATETIME_FORMAT)),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket.current_workflow_step,
                TicketWorkflowStep)),
            u'description': unicode(self.ticket.description),
            u'title': unicode(self.ticket.title),
            u'bug_id': self.ticket.bug_id,
            u'owner': unicode(self.ticket.owner),
            u'removed_binaries': self.ticket.removed_binaries,
            u'base_image': unicode(self.ticket.base_image),
            u'id': self.ticket.pk,
            u'master_ppa': unicode(self.ticket.master_ppa),
            u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                self.ticket.pk),
            u'series': unicode(self.ticket.series),
            u'updated': unicode(self.ticket.updated.strftime(
                settings.TEST_DATETIME_FORMAT)),
        })

    def test_get_sourcepackageupload_api(self):
        obj = self.getResource('spu/')
        self.assertEqual(obj['objects'][0], {
            u'version': unicode(self.spu.version),
            u'sourcepackage': {
                u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                    self.sourcepackage.pk),
                u'id': self.sourcepackage.pk,
                u'name': unicode(self.sourcepackage.name)},
            u'id': self.spu.pk,
            u'resource_uri':
            u'/api/v1/spu/{0}/'.format(self.spu.pk),
        })

    def test_get_ticket_update_status_api(self):
        obj = self.getResource('updateticketstatus/')
        self.assertEqual(obj['objects'][0], {
            u'id': self.ticket.pk,
            u'resource_uri':
            u'/api/v1/updateticketstatus/{0}/'.format(self.ticket.pk),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket.current_workflow_step,
                TicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.ticket.status,
                               TicketWorkflowStepStatus)),
        })

    def test_get_single_ticket_update_status_api(self):
        obj = self.getResource(
            'updateticketstatus/?id={}'.format(self.ticket.pk))
        self.assertEqual(obj['objects'][0], {
            u'id': self.ticket.pk,
            u'resource_uri':
            u'/api/v1/updateticketstatus/{0}/'.format(self.ticket.pk),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket.current_workflow_step,
                TicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.ticket.status,
                               TicketWorkflowStepStatus)),
        })

    def test_get_ticket_status_api(self):
        obj = self.getResource('ticketstatus/')
        self.assertEqual(obj['objects'][0], {
            u'created': unicode(self.ticket.created.strftime(
                settings.TEST_DATETIME_FORMAT)),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket.current_workflow_step,
                TicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.ticket.status,
                               TicketWorkflowStepStatus)),
            u'id': self.ticket.pk,
            u'title': self.ticket.title,
            u'resource_uri':
            u'/api/v1/ticketstatus/{0}/'.format(self.ticket.pk),
            u'updated': unicode(self.ticket.updated.strftime(
                settings.TEST_DATETIME_FORMAT)),
        })

    def test_get_subticket_update_status_api(self):
        obj = self.getResource('updatesubticketstatus/')
        self.assertEqual(obj['objects'][0], {
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/updatesubticketstatus/{0}/'.format(
                self.subticket.pk),
            u'current_workflow_step': unicode(get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.subticket.status,
                               SubTicketWorkflowStepStatus)),
        })

    def test_get_single_subticket_update_status_api(self):
        obj = self.getResource(
            'updatesubticketstatus/?id={}'.format(self.subticket.pk))
        self.assertEqual(obj['objects'][0], {
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/updatesubticketstatus/{0}/'.format(
                self.subticket.pk),
            u'current_workflow_step': unicode(get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.subticket.status,
                               SubTicketWorkflowStepStatus)),
        })

    def test_get_subticket_status_api(self):
        obj = self.getResource('subticketstatus/')
        self.assertEqual(obj['objects'][0], {
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/subticketstatus/{0}/'.format(
                self.subticket.pk),
            u'current_workflow_step': unicode(get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep)),
            u'status': unicode(get_enum_title(self.subticket.status,
                               SubTicketWorkflowStepStatus)),
        })

    def test_get_ticket_steps(self):
        obj = self.getResource('ticket_workflow_step/')
        self.assertEqual(len(obj['objects']),
                         len(ticket_states.TicketWorkflowStep))

    def test_get_subticket_steps(self):
        obj = self.getResource('subticket_workflow_step/')
        self.assertEqual(len(obj['objects']),
                         len(ticket_states.SubTicketWorkflowStep))

    def test_get_ticket_step_statuses(self):
        obj = self.getResource('ticket_workflow_step_status/')
        self.assertEqual(len(obj['objects']),
                         len(ticket_states.TicketWorkflowStepStatus))

    def test_get_subticket_step_statuses(self):
        obj = self.getResource('subticket_workflow_step_status/')
        self.assertEqual(len(obj['objects']),
                         len(ticket_states.SubTicketWorkflowStepStatus))

    def assert_same(self, uri):
        obj = self.getResource(uri)
        orig = obj['objects'][0]
        resource_uri = obj['objects'][0]['resource_uri']
        item_uri = resource_uri.replace('/api/v1/', '')

        obj = self.getResource(item_uri)
        self.assertEqual(orig, obj)

    def test_get_ticket_steps_detail(self):
        self.assert_same('ticket_workflow_step/')

    def test_get_subticket_steps_detail(self):
        self.assert_same('subticket_workflow_step/')

    def test_get_ticket_step_status_detail(self):
        self.assert_same('ticket_workflow_step_status/')

    def test_get_subticket_step_status_detail(self):
        self.assert_same('subticket_workflow_step_status/')


class GetOpenTicketsTest(TastypieTestCase):
    """
    Test that /opentickets/ returns only in progress and queued tickets.
    """

    def setUp(self):
        super(GetOpenTicketsTest, self).setUp('/api/v1')
        self.ticket_1 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.NEW))
        self.ticket_2 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.QUEUED))
        self.ticket_3 = mommy.make(
            'Ticket', current_workflow_step=int(
                TicketWorkflowStep.PKG_BUILDING))
        self.ticket_4 = mommy.make(
            'Ticket', current_workflow_step=int(
                TicketWorkflowStep.IMAGE_BUILDING))
        self.ticket_5 = mommy.make(
            'Ticket', current_workflow_step=int(
                TicketWorkflowStep.IMAGE_TESTING))
        self.ticket_6 = mommy.make(
            'Ticket', current_workflow_step=int(
                TicketWorkflowStep.PKG_PUBLISHING))
        self.ticket_7 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.COMPLETED))
        self.sourcepackage = sourcepackage_recipe.make()
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.subticket_1 = mommy.make('SubTicket', ticket=self.ticket_1,
                                      source_package_upload=self.spu)
        self.artifact_1 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_1)
        self.subticket_2 = mommy.make('SubTicket', ticket=self.ticket_2,
                                      source_package_upload=self.spu)
        self.artifact_2 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_2)
        self.subticket_3 = mommy.make('SubTicket', ticket=self.ticket_3,
                                      source_package_upload=self.spu)
        self.artifact_3 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_3)
        self.subticket_4 = mommy.make('SubTicket', ticket=self.ticket_4,
                                      source_package_upload=self.spu)
        self.artifact_4 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_4)
        self.subticket_5 = mommy.make('SubTicket', ticket=self.ticket_5,
                                      source_package_upload=self.spu)
        self.artifact_5 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_5)
        self.subticket_6 = mommy.make('SubTicket', ticket=self.ticket_6,
                                      source_package_upload=self.spu)
        self.artifact_6 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_6)
        self.subticket_7 = mommy.make('SubTicket', ticket=self.ticket_7,
                                      source_package_upload=self.spu)
        self.artifact_7 = mommy.make('SubTicketArtifact',
                                     subticket=self.subticket_7)

    def test_total_open_tickets(self):
        obj = self.getResource('opentickets/')
        self.assertEqual(len(obj['objects']), 5)

    def test_open_ticket_syntax(self):
        self.ticket_2.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_2.save()
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_3.save()
        self.ticket_4.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_4.save()
        self.ticket_5.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_5.save()
        obj = self.getResource('opentickets/')
        self.assertEqual(obj['objects'][0], {
            u'added_binaries': self.ticket_6.added_binaries,
            u'status': unicode(get_enum_title(self.ticket_6.status,
                               TicketWorkflowStepStatus)),
            u'created': unicode(self.ticket_6.created.strftime(
                settings.TEST_DATETIME_FORMAT)),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket_6.current_workflow_step,
                TicketWorkflowStep)),
            u'description': unicode(self.ticket_6.description),
            u'title': unicode(self.ticket_6.title),
            u'bug_id': self.ticket_6.bug_id,
            u'owner': unicode(self.ticket_6.owner),
            u'removed_binaries': self.ticket_6.removed_binaries,
            u'base_image': unicode(self.ticket_6.base_image),
            u'id': self.ticket_6.pk,
            u'master_ppa': unicode(self.ticket_6.master_ppa),
            u'resource_uri': u'/api/v1/opentickets/{0}/'.format(
                self.ticket_6.pk),
            u'series': unicode(self.ticket_6.series),
            u'updated': unicode(self.ticket_6.updated.strftime(
                settings.TEST_DATETIME_FORMAT)),
        })

    def test_no_open_tickets(self):
        self.ticket_2.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_2.save()
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_3.save()
        self.ticket_4.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_4.save()
        self.ticket_5.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_5.save()
        self.ticket_6.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_6.save()
        obj = self.getResource('opentickets/')
        self.assertEqual(len(obj['objects']), 0)
